//==- ModularizeHeaderInstance.cpp - Stores header instance info -*- C++ -*-==//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#include "ModularizeHeaderInstance.h"
#include "ModularizeUtilities.h"
#include "llvm/Support/raw_ostream.h"
#include "clang/Basic/SourceManager.h"

namespace Modularize {
using namespace llvm;
using namespace clang;

ModularizeHeaderInstance::ModularizeHeaderInstance(
	  ModularizeHeaderFile *HeaderFile, std::string topHeader) :
    HeaderFile(HeaderFile) {
  TopHeaders.push_back(topHeader);
}

ModularizeHeaderInstance::~ModularizeHeaderInstance() {
}

bool ModularizeHeaderInstance::HasTopHeader(std::string topHeader) {
  ModularizeTopHeaderVectorIterator iter = TopHeaders.begin();
  ModularizeTopHeaderVectorIterator end = TopHeaders.begin();
  for (; iter != end; ++iter) {
    if (*iter == topHeader)
      return true;
  }
  return false;
}

// Display the information for debugging.
void ModularizeHeaderInstance::Dump(int level) {
  errs() << Indent(level) << "top headers:\n";

	ModularizeTopHeaderVectorIterator thIter = GetTopHeadersBegin();
	ModularizeTopHeaderVectorIterator thEnd = GetTopHeadersEnd();
  for (; thIter != thEnd; ++thIter) {
    errs() << Indent(level + 1) << *thIter << "\n";
  }

  errs() << Indent(level) << "preprocessor directives: \n";
	ModularizePPDirectiveVectorIterator dIter = HeaderFile->GetDirectivesBegin();
	ModularizePPDirectiveVectorIterator dEnd = HeaderFile->GetDirectivesEnd();
  for (; dIter != dEnd; ++dIter) {
    (*dIter).Dump(level + 1, HeaderFile->GetName().c_str());
  }
}

}
