//===--- ModularizeHeaderFile.cpp - Callbacks for Preprocessor actions ---*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#include "ModularizeHeaderFile.h"
#include "ModularizeUtilities.h"
#include "llvm/Support/raw_ostream.h"
#include "clang/Basic/SourceManager.h"

namespace Modularize {
using namespace llvm;
using namespace clang;

ModularizeHeaderFile::ModularizeHeaderFile(std::string name, Preprocessor& pp) :
  Name(name) {
}

ModularizeHeaderFile::~ModularizeHeaderFile() {
}

// Get highest line-numbered directive.
unsigned ModularizeHeaderFile::GetLastLineNumber() {
  if (Directives.size() == 0)
    return 0;
  return Directives[Directives.size() - 1].GetLineNumber();
}

// Return true if this snapshot matches the other snapshot.
bool ModularizeHeaderFile::Match(ModularizeHeaderFile *other) {
  bool returnValue = true;
  // Walk conditional directives.  If the condition
  // (after macro substitutions) is not the same,
  // return false, meaning the header file snapshot doesn't match.
	ModularizePPDirectiveVectorIterator thisIter = GetDirectivesBegin();
	ModularizePPDirectiveVectorIterator thisEnd = GetDirectivesEnd();
	ModularizePPDirectiveVectorIterator otherIter = other->GetDirectivesBegin();
	ModularizePPDirectiveVectorIterator otherEnd = other->GetDirectivesEnd();
  for (; (thisIter != thisEnd) && (otherIter != otherEnd);
      ++thisIter, ++ otherIter) {
    if (!(*thisIter).Match(&(*otherIter))) {
      returnValue = false;
      break;
    }
  }
  return returnValue;
}

// Display the information for debugging.
void ModularizeHeaderFile::Dump(int level) {
  errs() << Indent(level) << "preprocessor directives:\n";
	ModularizePPDirectiveVectorIterator dIter = GetDirectivesBegin();
	ModularizePPDirectiveVectorIterator dEnd = GetDirectivesEnd();
  for (; dIter != dEnd; ++dIter) {
    (*dIter).Dump(level + 1, Name.c_str());
  }
}

}
