//===- ModularizeHeaderTracker.h - Represents header instances -*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
///
/// \file
/// \brief Defines a class to store header instance snapshot information
///   for one header file.
///
//===----------------------------------------------------------------------===//

#ifndef MODULARIZEHEADERTRACKER_H
#define MODULARIZEHEADERTRACKER_H

#include "ModularizeHeaderInstance.h"

namespace Modularize {
  using namespace clang;
  using namespace llvm;

/// \brief This class stores the header snapshot instances for a particular
///   header.
class ModularizeHeaderTracker {
public:
  ModularizeHeaderTracker(std::string& name, ModularizeHeaderFile *headerFile,
	  std::string topHeader);
  ~ModularizeHeaderTracker();

  // Header name accessors.
  std::string& GetName() { return Name; }
  void SetName(std::string name) { Name = name; };

  // Process a header snapshot.
  // If the snapshot differs, add a new header instance.
  void AddHeaderFile(ModularizeHeaderFile *headerFile, std::string topHeader);
  // Add a header instance.
  void AddHeaderInstance(ModularizeHeaderInstance *headerInstance);
  // Get instance count.  > 1 means there were mismatches.
  int GetHeaderInstanceCount() { return HeaderInstances.size(); }
  // Get header instance vector.
  ModularizeHeaderInstanceVector &GetHeaderInstances() { return HeaderInstances; }
  ModularizeHeaderInstanceVectorIterator GetHeaderInstancesBegin() { return HeaderInstances.begin(); }
  ModularizeHeaderInstanceVectorIterator GetHeaderInstancesEnd() { return HeaderInstances.end(); }
  ModularizeHeaderInstance *GetHeaderInstance(int index) { return HeaderInstances[index]; }

  // Report instances where header snapshots differ.
  // Returns true if no errors, i.e. no differing snapshots.
  bool Report();
  // Display the information for debugging.
  void Dump(int level = 0);

private:
  std::string Name;
  ModularizeHeaderInstanceVector HeaderInstances;
};

typedef StringMap<ModularizeHeaderTracker *> ModularizeHeaderTrackerMap;
typedef ModularizeHeaderTrackerMap::iterator ModularizeHeaderTrackerMapIterator;

}  // end namespace clang

#endif
